from os import environ, getenv
from pathlib import Path

from promptquality.constants.config import ConfigDefaults, ConfigEnvironmentVariables


def _config_location() -> Path:
    return Path.home().joinpath(
        ConfigDefaults.config_directory, ConfigDefaults.config_filename
    )


def set_console_url(console_url: str) -> None:
    """For Enterprise users. Set the console URL to your Galileo Environment.

    You can also set GALILEO_CONSOLE_URL before importing promptquality to bypass this

    :param console_url: If set, that will be used. Otherwise, if an environment variable
    GALILEO_CONSOLE_URL is set, that will be used. Otherwise, you will be prompted for
    a url.
    """
    if getenv(ConfigEnvironmentVariables.console_url, None) is None:
        environ[ConfigEnvironmentVariables.console_url] = console_url



from types import SimpleNamespace

ConfigDefaults = SimpleNamespace(
    console_url="https://console.cloud.rungalileo.io/",
    config_directory=".galileo",
    config_filename="pq-config.json",
)


ConfigEnvironmentVariables = SimpleNamespace(
    console_url="GALILEO_CONSOLE_URL",
    username="GALILEO_USERNAME",
    password="GALILEO_PASSWORD",
)



from typing import List, Optional

from pydantic import BaseModel, ConfigDict


class Settings(BaseModel):
    """
    Settings for a prompt run that a user can configure.
    """

    model_alias: Optional[str]
    temperature: Optional[float]
    max_tokens: Optional[int]
    stop_sequences: Optional[List[str]]

    top_p: Optional[int]
    frequency_penalty: Optional[float]
    presence_penalty: Optional[float]

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from promptquality.constants.config import ConfigDefaults
from promptquality.types.config import Config
from promptquality.utils.config import _config_location, set_console_url


def set_config(console_url: Optional[str] = None) -> Config:
    """
    Set the config for `promptquality`.

    If the config file exists, and console_url is not passed, read it and return the
    config. Otherwise, set the default console URL and return the config.

    Parameters
    ----------
    console_url : Optional[str], optional
        URL to the Galileo console, by default None and we use the Galileo Cloud URL.

    Returns
    -------
    Config
        Config object for `promptquality`.
    """
    if not console_url and _config_location().exists():
        config = Config.read()
    else:
        console_url = console_url or ConfigDefaults.console_url
        set_console_url(console_url=console_url)
        config = Config(console_url=console_url)
    config.write()
    return config



from posixpath import join
from typing import Any, Callable, Dict, List, Optional, Union

from pydantic import UUID4, BaseModel, HttpUrl, SecretStr
from requests import get, post, put

from promptquality.constants.routes import Routes
from promptquality.types.run import (
    CreateIntegrationRequest,
    CreateJobRequest,
    CreateProjectRequest,
    CreateRunRequest,
    CreateTemplateRequest,
    GetMetricsRequest,
    UploadDatasetRequest,
)
from promptquality.utils.request import HttpHeaders, make_request


class ApiClient(BaseModel):
    api_url: HttpUrl
    token: SecretStr

    @staticmethod
    def healthcheck(base_url: str) -> bool:
        make_request(get, base_url=base_url, endpoint=Routes.healthcheck)
        return True

    @staticmethod
    def username_login(base_url: str, username: str, password: str) -> Dict[str, str]:
        return make_request(
            post,
            base_url=base_url,
            endpoint=Routes.login,
            data={
                "username": username,
                "password": password,
                "auth_method": "email",
            },
        )

    @property
    def base_url(self) -> str:
        return self.api_url.unicode_string()

    @property
    def auth_header(self) -> Dict[str, str]:
        return {"Authorization": f"Bearer {self.token.get_secret_value()}"}

    def _make_request(
        self,
        request_method: Callable,
        endpoint: str,
        body: Optional[Dict] = None,
        data: Optional[Dict] = None,
        files: Optional[Dict] = None,
        params: Optional[Dict] = None,
        timeout: Union[int, None] = None,
        json_request_only: bool = False,
    ) -> Any:
        if json_request_only:
            content_headers = HttpHeaders.accept_json()
        else:
            content_headers = HttpHeaders.json()
        headers = {**self.auth_header, **content_headers}
        return make_request(
            request_method=request_method,
            base_url=self.base_url,
            endpoint=endpoint,
            body=body,
            data=data,
            files=files,
            params=params,
            headers=headers,
            timeout=timeout,
        )

    def get_current_user(self) -> Dict[str, str]:
        return self._make_request(get, endpoint=Routes.current_user)

    def create_project(self, project_request: CreateProjectRequest) -> Dict[str, str]:
        return self._make_request(
            post,
            endpoint=Routes.projects,
            body=project_request.model_dump(),
        )

    def create_template(
        self, template_request: CreateTemplateRequest
    ) -> Dict[str, str]:
        return self._make_request(
            post,
            endpoint=Routes.templates.format(project_id=template_request.project_id),
            body=template_request.model_dump(mode="json"),
        )

    def upload_dataset(self, dataset_request: UploadDatasetRequest) -> Dict[str, str]:
        return self._make_request(
            post,
            json_request_only=True,
            endpoint=Routes.dataset.format(project_id=dataset_request.project_id),
            data=dataset_request.data,
            files=dataset_request.files,
        )

    def create_run(self, run_request: CreateRunRequest) -> Dict[str, str]:
        return self._make_request(
            post,
            endpoint=Routes.runs.format(project_id=run_request.project_id),
            body=run_request.model_dump(mode="json"),
        )

    def create_job(self, job_request: CreateJobRequest) -> Dict[str, str]:
        return self._make_request(
            post,
            endpoint=Routes.jobs,
            body=job_request.model_dump(mode="json", exclude_none=True),
        )

    def get_metrics(self, metrics_request: GetMetricsRequest) -> List[Dict[str, Any]]:
        return self._make_request(
            get,
            endpoint=Routes.metrics.format(**metrics_request.model_dump(mode="json")),
        )

    def get_job_status(self, job_id: UUID4) -> Dict[str, Any]:
        return self._make_request(get, endpoint=join(Routes.jobs, str(job_id)))

    def put_integration(
        self, integration_request: CreateIntegrationRequest
    ) -> Dict[str, Any]:
        return self._make_request(
            put,
            endpoint=Routes.integrations.format(
                integration_name=integration_request.name
            ),
            body=integration_request.body,
        )



from pathlib import Path
from typing import Optional

from pydantic import UUID4

from promptquality.set_config import set_config
from promptquality.types.config import Config
from promptquality.types.run import (
    CreateJobRequest,
    CreateJobResponse,
    CreateProjectRequest,
    CreateProjectResponse,
    CreateRunRequest,
    CreateRunResponse,
    CreateTemplateRequest,
    CreateTemplateResponse,
    GetJobStatusResponse,
    UploadDatasetRequest,
    UploadDatasetResponse,
)
from promptquality.types.settings import Settings
from promptquality.utils.logger import logger


def create_project(
    project_name: Optional[str] = None, config: Optional[Config] = None
) -> UUID4:
    config = config or set_config()
    project_request = CreateProjectRequest(name=project_name)
    logger.debug(f"Creating project {project_request.name}...")
    response_dict = config.api_client.create_project(project_request)
    project_response = CreateProjectResponse.model_validate(response_dict)
    config.merge_project(project_response)
    logger.debug(
        f"Created project with name {project_response.name}, ID {project_response.id}."
    )
    return project_response.id


def create_template(
    template: str,
    project_id: UUID4,
    template_name: Optional[str] = None,
    config: Optional[Config] = None,
) -> UUID4:
    config = config or set_config()
    template_request = CreateTemplateRequest(
        template=template, project_id=project_id, name=template_name
    )
    logger.debug(f"Creating template {template_request.name}...")
    response_dict = config.api_client.create_template(template_request)
    template_response = CreateTemplateResponse.model_validate(response_dict)
    config.merge_template(template_response)
    logger.debug(
        f"Created template with name {template_response.name}, ID "
        f"{template_response.id}."
    )
    return template_response.selected_version_id


def upload_dataset(
    dataset: Path,
    project_id: UUID4,
    template_version_id: UUID4,
    config: Optional[Config] = None,
) -> UUID4:
    config = config or set_config()
    dataset_request = UploadDatasetRequest.from_dataset(
        dataset, project_id, template_version_id
    )
    logger.debug(f"Uploading dataset {dataset_request.file_path}...")
    response_dict = config.api_client.upload_dataset(dataset_request)
    dataset_response = UploadDatasetResponse.model_validate(response_dict)
    config.merge_dataset(dataset_response)
    logger.debug(f"Uploaded dataset with ID {dataset_response.id}.")
    return dataset_response.id


def create_run(
    project_id: UUID4, run_name: Optional[str] = None, config: Optional[Config] = None
) -> UUID4:
    config = config or set_config()
    run_request = CreateRunRequest(name=run_name, project_id=project_id)
    logger.debug(f"Creating run {run_request.name}...")
    response_dict = config.api_client.create_run(run_request)
    run_response = CreateRunResponse.model_validate(response_dict)
    config.merge_run(run_response)
    logger.debug(f"Created run with name {run_request.name}, ID {run_response.id}.")
    return run_response.id


def create_job(
    project_id: UUID4,
    run_id: UUID4,
    dataset_id: UUID4,
    template_version_id: UUID4,
    settings: Optional[Settings] = None,
    config: Optional[Config] = None,
) -> UUID4:
    config = config or set_config()
    job_request = CreateJobRequest(
        project_id=project_id,
        run_id=run_id,
        prompt_dataset_id=dataset_id,
        prompt_settings=settings,
        prompt_template_version_id=template_version_id,
    )
    logger.debug("Creating job...")
    response_dict = config.api_client.create_job(job_request)
    job_response = CreateJobResponse.model_validate(response_dict)
    config.merge_job(job_response)
    logger.debug(f"Created job with ID {job_response.id}.")
    return job_response.id


def get_job_status(
    job_id: Optional[UUID4] = None, config: Optional[Config] = None
) -> GetJobStatusResponse:
    config = config or set_config()
    logger.debug(f"Getting job status for job {job_id}...")
    job_id = job_id or config.current_job_id
    if not job_id:
        raise ValueError("job_id must be provided")
    response_dict = config.api_client.get_job_status(job_id)
    job_status_response = GetJobStatusResponse.model_validate(response_dict)
    logger.debug(
        f"Got job status for job {job_id}, status is "
        f"{job_status_response.progress_message}, "
        f"{job_status_response.progress_percent} percentage."
    )
    return job_status_response



"""PromptQuality"""
# flake8: noqa F401

from promptquality.get_metrics import get_metrics
from promptquality.integrations import add_openai_integration
from promptquality.job_progress import job_progress
from promptquality.login import login
from promptquality.run import run
from promptquality.set_config import set_config
from promptquality.types.settings import Settings

__version__ = "0.2.1"
